"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var path_1 = tslib_1.__importDefault(require("path"));
var speed_measure_webpack_plugin_1 = tslib_1.__importDefault(require("speed-measure-webpack-plugin"));
var build_1 = require("../build/build");
var clean_1 = tslib_1.__importDefault(require("../clean"));
var config_1 = require("../runtime/config/config");
var fs_1 = require("../runtime/util/fs");
var webpack_1 = require("../util/webpack");
var cache_validator_1 = require("./cache-validator");
var ts_loader_config_1 = require("../config/ts-loader-config");
function smp(buildConfig, webpackConfig) {
    var smpPlugin = new speed_measure_webpack_plugin_1.default();
    return buildConfig.SMP ? smpPlugin.wrap(webpackConfig) : webpackConfig;
}
exports.smp = smp;
var buildTarget = function (log, buildConfig, target, environment) {
    if (environment === void 0) { environment = 'prod'; }
    return tslib_1.__awaiter(void 0, void 0, void 0, function () {
        var config, disableCaching, webpackConfigString, configHash, validationItems, yarnLockPath, npmLockPath, babelConfig;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    config = build_1.getWebpackConfig(log, buildConfig, target, environment);
                    if (!config) {
                        return [2 /*return*/, Promise.reject("Could not load webpack configuration for " + target + "/" + environment + "!")];
                    }
                    disableCaching = String(process.env.BUILD_CACHE_DISABLED).toLowerCase() === 'true';
                    if (!disableCaching) return [3 /*break*/, 1];
                    log.info("Build Caching Disabled... BUILD_CACHE_DISABLED=" + disableCaching);
                    return [3 /*break*/, 4];
                case 1:
                    if (!(process.env.NODE_ENV !== 'test' && !disableCaching)) return [3 /*break*/, 4];
                    webpackConfigString = JSON.stringify(config);
                    return [4 /*yield*/, cache_validator_1.getMd5(log, 'webpackConfig', webpackConfigString)];
                case 2:
                    configHash = _a.sent();
                    validationItems = [
                        {
                            filePath: ts_loader_config_1.getTsConfigFile(target, buildConfig),
                            hashKey: 'tsconfigHash',
                            isFile: true,
                        },
                        { isFile: false, itemHash: configHash, hashKey: 'webpackConfig' },
                    ];
                    yarnLockPath = path_1.default.resolve(process.cwd(), 'yarn.lock');
                    if (fs_1.existsSync(yarnLockPath)) {
                        validationItems.push({
                            filePath: yarnLockPath,
                            hashKey: 'yarnLockConfigHash',
                            isFile: true,
                        });
                    }
                    npmLockPath = path_1.default.resolve(process.cwd(), 'package-lock.json');
                    if (fs_1.existsSync(npmLockPath)) {
                        validationItems.push({
                            filePath: npmLockPath,
                            hashKey: 'npmLockConfigHash',
                            isFile: true,
                        });
                    }
                    babelConfig = ts_loader_config_1.getBabelConfigFile(target, buildConfig);
                    if (babelConfig && fs_1.existsSync(babelConfig)) {
                        validationItems.push({
                            filePath: babelConfig,
                            hashKey: 'babelConfigHash',
                            isFile: true,
                        });
                    }
                    cache_validator_1.setupWithBuildInfo(log, {
                        buildInfo: {
                            environment: environment,
                            project: buildConfig.BASE,
                            target: target,
                        },
                        traceMessages: false,
                        validationItems: validationItems,
                    });
                    return [4 /*yield*/, cache_validator_1.validateCache(log)];
                case 3:
                    _a.sent();
                    _a.label = 4;
                case 4: return [2 /*return*/, webpack_1.webpackPromise(log, smp(buildConfig, config)).then(function () {
                        var runtimeConfig = {
                            ASSETS_PATH: buildConfig.ASSETS_PATH_PREFIX,
                            ASSETS_PATH_PREFIX: buildConfig.ASSETS_PATH_PREFIX,
                            BASE: '.',
                            PUBLIC_PATH: buildConfig.PUBLIC_PATH,
                            SERVER_PUBLIC_DIR: buildConfig.SERVER_PUBLIC_DIR === false ? false : 'public/',
                        };
                        // On success write out watchtower config
                        return fs_1.writeFile(log, path_1.default.join(buildConfig.OUTPUT, config_1.watchtowerConfigFilename), JSON.stringify(runtimeConfig, undefined, 2));
                    })];
            }
        });
    });
};
var getBuildEnvironment = function (args) {
    for (var _i = 0, args_1 = args; _i < args_1.length; _i++) {
        var arg = args_1[_i];
        if (typeof arg !== 'string') {
            continue;
        }
        if (build_1.ENVIRONMENTS.indexOf(arg) !== -1) {
            return arg;
        }
    }
    return 'prod';
};
var getBuildTargets = function (buildConfig, args) {
    for (var _i = 0, args_2 = args; _i < args_2.length; _i++) {
        var arg = args_2[_i];
        if (!buildConfig.HAS_SERVER && arg === 'server') {
            continue;
        }
        if (typeof arg !== 'string') {
            continue;
        }
        if (build_1.TARGETS.indexOf(arg) !== -1) {
            return [arg];
        }
    }
    var defaultTargets = buildConfig.HAS_SERVER ? ['server', 'client'] : ['client'];
    return defaultTargets;
};
/**
 * Builds one or all targets in a specified environment
 * @param args
 *  [<target>] [<environment>] [-p <project root>]
 *  - target defaults to both server and client
 *  - environment defaults to prod
 *  - p: the root of the build, used for config discovery
 */
var build = function (log, buildConfig) {
    var args = [];
    for (var _i = 2; _i < arguments.length; _i++) {
        args[_i - 2] = arguments[_i];
    }
    return tslib_1.__awaiter(void 0, void 0, void 0, function () {
        var targets, environment, cleanTarget, _a, targets_1, target;
        return tslib_1.__generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    targets = getBuildTargets(buildConfig, args);
                    environment = getBuildEnvironment(args);
                    cleanTarget = buildConfig.OUTPUT;
                    return [4 /*yield*/, clean_1.default(log, cleanTarget)];
                case 1:
                    _b.sent();
                    _a = 0, targets_1 = targets;
                    _b.label = 2;
                case 2:
                    if (!(_a < targets_1.length)) return [3 /*break*/, 5];
                    target = targets_1[_a];
                    return [4 /*yield*/, buildTarget(log, buildConfig, target, environment)];
                case 3:
                    _b.sent();
                    _b.label = 4;
                case 4:
                    _a++;
                    return [3 /*break*/, 2];
                case 5: return [2 /*return*/];
            }
        });
    });
};
exports.default = build;
//# sourceMappingURL=build.js.map