"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var fs_1 = tslib_1.__importDefault(require("fs"));
var md5_1 = tslib_1.__importDefault(require("md5"));
var mkdirp_1 = tslib_1.__importDefault(require("mkdirp"));
var path_1 = tslib_1.__importDefault(require("path"));
var rimraf_1 = tslib_1.__importDefault(require("rimraf"));
var util_1 = require("util");
// Configure Variables
var cacheDir = process.env.BUILD_CACHE_DIRECTORY || ".build-cache";
var cacheDirPath = path_1.default.join(path_1.default.resolve(cacheDir));
var buildCacheValidationConfig = '.build-cache-validation';
// default values
var validatorConfig = {
    baseCacheDirectory: cacheDirPath,
    cacheDirectory: cacheDirPath,
    cacheValidationConfigPath: path_1.default.join(cacheDirPath, buildCacheValidationConfig),
    validationItems: [],
};
var TRACE_MESSAGES = true;
var traceLog = function (log, message) {
    if (TRACE_MESSAGES) {
        log.trace(message);
    }
};
exports.setup = function (log, options) {
    var cacheDirectory = options.cacheDirectory, validationItems = options.validationItems, valiationConfigPath = options.valiationConfigPath, traceMessages = options.traceMessages;
    if (traceMessages !== undefined) {
        TRACE_MESSAGES = traceMessages;
    }
    if (cacheDirectory) {
        validatorConfig.cacheDirectory = cacheDirectory;
        validatorConfig.baseCacheDirectory = cacheDirectory;
    }
    if (validationItems) {
        validatorConfig.validationItems = validationItems;
    }
    if (valiationConfigPath) {
        validatorConfig.cacheValidationConfigPath = valiationConfigPath;
    }
    traceLog(log, "[Cache-Validator] validatorConfig => " + JSON.stringify(validatorConfig));
    return validatorConfig;
};
exports.setupWithBuildInfo = function (log, options) {
    var cacheDirectory = options.cacheDirectory, buildInfo = options.buildInfo, validationItems = options.validationItems, traceMessages = options.traceMessages;
    var project = buildInfo.project, environment = buildInfo.environment, target = buildInfo.target;
    if (traceMessages !== undefined) {
        TRACE_MESSAGES = traceMessages;
    }
    if (cacheDirectory) {
        validatorConfig.cacheDirectory = cacheDirectory;
        validatorConfig.baseCacheDirectory = cacheDirectory;
    }
    // Project Environment Target Folder
    var petFolder = path_1.default.join(project.replace('/', '.'), environment + "." + target);
    var newCacheDir = path_1.default.join(validatorConfig.baseCacheDirectory, petFolder);
    // Update config due to petFolder (pet = project environment target)
    validatorConfig.cacheDirectory = newCacheDir;
    validatorConfig.cacheValidationConfigPath = path_1.default.join(newCacheDir, '.build-cache-validation');
    if (validationItems) {
        validatorConfig.validationItems = validationItems;
    }
    traceLog(log, "[Cache-Validator] validatorConfig => " + JSON.stringify(validatorConfig));
    return validatorConfig;
};
exports.buildCacheDirectory = function (buildInfo) {
    var project = buildInfo.project, environment = buildInfo.environment, target = buildInfo.target;
    // Project Environment Target Folder
    var petFolder = path_1.default.join(project.replace('/', '.'), environment + "." + target);
    return path_1.default.join(validatorConfig.baseCacheDirectory, petFolder);
};
var writeFile = util_1.promisify(fs_1.default.writeFile);
var rmrf = util_1.promisify(rimraf_1.default);
var fileExists = util_1.promisify(fs_1.default.exists);
var getFileContents = function (log, filePath) { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
    var readFile, exists, contents;
    return tslib_1.__generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                readFile = util_1.promisify(fs_1.default.readFile);
                return [4 /*yield*/, fileExists(filePath)];
            case 1:
                exists = _a.sent();
                if (!exists) return [3 /*break*/, 3];
                return [4 /*yield*/, readFile(filePath, { encoding: 'utf8' })];
            case 2:
                contents = _a.sent();
                traceLog(log, "[Cache-Validator] contents from " + filePath + " is '" + contents + "'");
                return [2 /*return*/, contents];
            case 3:
                traceLog(log, "[Cache-Validator] " + filePath + " does not exist, no contents returned");
                return [2 /*return*/, null];
        }
    });
}); };
exports.getConfigContents = function (log, configFilePath) { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
    var result, cast;
    return tslib_1.__generator(this, function (_a) {
        switch (_a.label) {
            case 0: return [4 /*yield*/, getFileContents(log, configFilePath)];
            case 1:
                result = _a.sent();
                if (result) {
                    cast = JSON.parse(result);
                    return [2 /*return*/, cast];
                }
                return [2 /*return*/, null];
        }
    });
}); };
var writeContents = function (log, filePath, contents) { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
    var jsonStr, exists, parentDir;
    return tslib_1.__generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                jsonStr = JSON.stringify(contents);
                traceLog(log, "[Cache-Validator] saving cacheLoaderValidation =>" + jsonStr);
                return [4 /*yield*/, fileExists(filePath)];
            case 1:
                exists = _a.sent();
                if (!exists) {
                    parentDir = path_1.default.dirname(filePath);
                    mkdirp_1.default.sync(parentDir);
                }
                return [4 /*yield*/, writeFile(filePath, jsonStr, { encoding: 'utf8', flag: 'w+' })];
            case 2:
                _a.sent();
                return [2 /*return*/];
        }
    });
}); };
exports.deletePath = function (log, fileOrDir) { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
    return tslib_1.__generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                traceLog(log, "[Cache-Validator] rmrf " + fileOrDir);
                return [4 /*yield*/, rmrf(fileOrDir)];
            case 1:
                _a.sent();
                return [2 /*return*/];
        }
    });
}); };
var clearDirectory = function (log, directory) { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
    var exists;
    return tslib_1.__generator(this, function (_a) {
        switch (_a.label) {
            case 0: return [4 /*yield*/, fileExists(directory)];
            case 1:
                exists = _a.sent();
                if (exists) {
                    log.info("[Cache-Validator] Clearing => " + directory);
                }
                else {
                    log.info("[Cache-Validator] " + directory + " not cleared (Does not exist)");
                }
                return [4 /*yield*/, exports.deletePath(log, directory)];
            case 2:
                _a.sent();
                return [2 /*return*/];
        }
    });
}); };
var getMd5OfFile = function (log, key, file) { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
    var tsConfig, md5Hash;
    return tslib_1.__generator(this, function (_a) {
        switch (_a.label) {
            case 0: return [4 /*yield*/, getFileContents(log, file)];
            case 1:
                tsConfig = _a.sent();
                if (tsConfig) {
                    md5Hash = md5_1.default(tsConfig);
                    traceLog(log, "[Cache-Validator] md5 hash for file => " + key + " contents -> " + md5Hash);
                    return [2 /*return*/, md5Hash];
                }
                return [2 /*return*/, null];
        }
    });
}); };
exports.getMd5 = function (log, key, item) { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
    var md5Hash;
    return tslib_1.__generator(this, function (_a) {
        md5Hash = md5_1.default(item);
        log.info("[Cache-Validator] md5 hash for " + key + " -> " + md5Hash);
        return [2 /*return*/, md5Hash];
    });
}); };
var shouldClearCache = function (log) { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
    var cacheLoaderValidation, config_1, result_1;
    return tslib_1.__generator(this, function (_a) {
        switch (_a.label) {
            case 0: return [4 /*yield*/, exports.getConfigContents(log, validatorConfig.cacheValidationConfigPath)];
            case 1:
                cacheLoaderValidation = _a.sent();
                if (!cacheLoaderValidation) return [3 /*break*/, 5];
                config_1 = cacheLoaderValidation;
                if (!(validatorConfig.validationItems.length > 0)) return [3 /*break*/, 5];
                result_1 = false;
                return [4 /*yield*/, Promise.all(validatorConfig.validationItems.map(function (validationItem) { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
                        var hash, _a;
                        return tslib_1.__generator(this, function (_b) {
                            switch (_b.label) {
                                case 0:
                                    traceLog(log, "[Cache-Validator] processing validation Item " + JSON.stringify(validationItem));
                                    if (!validationItem.isFile) return [3 /*break*/, 2];
                                    return [4 /*yield*/, getMd5OfFile(log, validationItem.hashKey, validationItem.filePath)];
                                case 1:
                                    _a = _b.sent();
                                    return [3 /*break*/, 3];
                                case 2:
                                    _a = validationItem.itemHash;
                                    _b.label = 3;
                                case 3:
                                    hash = _a;
                                    if (hash !== config_1[validationItem.hashKey]) {
                                        log.info("[Cache-Validator] " + validationItem.hashKey + " hashMismatch, config=" + config_1[validationItem.hashKey] + " " + (validationItem.isFile ? 'file' : 'object') + "=" + hash);
                                        result_1 = true;
                                    }
                                    return [2 /*return*/];
                            }
                        });
                    }); }))];
            case 2:
                _a.sent();
                if (!!result_1) return [3 /*break*/, 4];
                log.info("[Cache-Validator] No mismatches found - cache not cleared");
                config_1.cacheCleared = false;
                return [4 /*yield*/, writeContents(log, validatorConfig.cacheValidationConfigPath, config_1)];
            case 3:
                _a.sent();
                _a.label = 4;
            case 4: return [2 /*return*/, result_1];
            case 5:
                traceLog(log, "[Cache-Validator] " + validatorConfig.cacheValidationConfigPath + " not found (Does not exist)");
                return [2 /*return*/, true];
        }
    });
}); };
var createValidationConfig = function (log, cacheCleared) { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
    var configToWrite;
    return tslib_1.__generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                configToWrite = { cacheCleared: cacheCleared };
                return [4 /*yield*/, Promise.all(validatorConfig.validationItems.map(function (validationItem) { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
                        var hashValue, _a;
                        return tslib_1.__generator(this, function (_b) {
                            switch (_b.label) {
                                case 0:
                                    if (!validationItem.isFile) return [3 /*break*/, 2];
                                    return [4 /*yield*/, getMd5OfFile(log, validationItem.hashKey, validationItem.filePath)];
                                case 1:
                                    _a = _b.sent();
                                    return [3 /*break*/, 3];
                                case 2:
                                    _a = validationItem.itemHash;
                                    _b.label = 3;
                                case 3:
                                    hashValue = _a;
                                    if (hashValue) {
                                        configToWrite[validationItem.hashKey] = hashValue;
                                    }
                                    return [2 /*return*/];
                            }
                        });
                    }); }))];
            case 1:
                _a.sent();
                log.info("[Cache-Validator] writing config file => " + validatorConfig.cacheValidationConfigPath);
                return [4 /*yield*/, writeContents(log, validatorConfig.cacheValidationConfigPath, configToWrite)];
            case 2:
                _a.sent();
                return [2 /*return*/];
        }
    });
}); };
/**
 * Determines if the build cache should be cleared if the ts config has changed.
 */
exports.validateCache = function (log) { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
    var clear;
    return tslib_1.__generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                log.info("[Cache-Validator] cacheDirectory => " + validatorConfig.cacheDirectory);
                return [4 /*yield*/, shouldClearCache(log)];
            case 1:
                clear = _a.sent();
                if (!clear) return [3 /*break*/, 4];
                return [4 /*yield*/, clearDirectory(log, validatorConfig.cacheDirectory)];
            case 2:
                _a.sent();
                return [4 /*yield*/, createValidationConfig(log, true)];
            case 3:
                _a.sent();
                _a.label = 4;
            case 4: return [2 /*return*/];
        }
    });
}); };
exports.writeDummyConfigFile = function (log, file, config) { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
    var contents;
    return tslib_1.__generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                contents = JSON.stringify(config);
                traceLog(log, "[Cache-Validator] writing dummy config " + file + " with contents " + contents);
                return [4 /*yield*/, writeFile(file, contents, 'utf8')];
            case 1:
                _a.sent();
                return [2 /*return*/];
        }
    });
}); };
exports.writeDummyFile = function (log, file, contents) { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
    return tslib_1.__generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                traceLog(log, "[Cache-Validator] writing dummy file " + file + " with contents " + contents);
                return [4 /*yield*/, writeFile(file, contents, 'utf8')];
            case 1:
                _a.sent();
                return [2 /*return*/];
        }
    });
}); };
//# sourceMappingURL=cache-validator.js.map