"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var build_1 = tslib_1.__importDefault(require("./build"));
var clean_1 = tslib_1.__importDefault(require("./clean"));
var explore_bundle_1 = tslib_1.__importDefault(require("./explore-bundle"));
var start_1 = tslib_1.__importDefault(require("./start"));
var watch_1 = tslib_1.__importDefault(require("./watch"));
var config_1 = require("../runtime/config/config");
var log_1 = require("../util/log");
var args = process.argv.slice(2);
var command = args[0];
var commandArgs = args.slice(1);
var exitOnError = function (result) {
    result.catch(function (e) {
        log_1.cliLogger.error({ err: e }, 'Error occurred, exiting...');
        process.exit(1);
    });
};
var exitAfter = function (result) {
    return exitOnError(result.then(function () {
        // force process to exit
        process.exit(0);
    }));
};
var projectSwitchIndex = commandArgs.indexOf('-p');
var workingDirectory = process.cwd();
if (projectSwitchIndex !== -1) {
    var projectSwitchArgs = commandArgs.splice(projectSwitchIndex, 2);
    if (projectSwitchArgs.length === 2) {
        workingDirectory = projectSwitchArgs[1];
    }
}
var buildConfig = config_1.getConfig(log_1.cliLogger, workingDirectory);
switch (command) {
    case 'build':
        exitAfter(build_1.default.apply(void 0, tslib_1.__spreadArrays([log_1.cliLogger, buildConfig], commandArgs)));
        break;
    case 'clean':
        exitAfter(clean_1.default.apply(void 0, tslib_1.__spreadArrays([log_1.cliLogger, buildConfig], commandArgs)));
        break;
    case 'explore-bundle':
        exitOnError(explore_bundle_1.default.apply(void 0, tslib_1.__spreadArrays([log_1.cliLogger, buildConfig], commandArgs)));
        break;
    case 'start':
        exitOnError(start_1.default.apply(void 0, tslib_1.__spreadArrays([log_1.cliLogger, buildConfig, {}], commandArgs)));
        break;
    case 'watch':
        exitOnError(watch_1.default.apply(void 0, tslib_1.__spreadArrays([log_1.cliLogger, buildConfig, {}], commandArgs)));
        break;
    default:
        log_1.cliLogger.info("\n## Project Watchtower\n\nScripts:\n\n    build [<target>] [<environment>] [-p <project dir>]\n    clean [-p <project dir>] [<glob> ...]\n    explore-bundle [disableHoisting] [-p <project dir>]\n    start [watch] [prod] [debug] [client] [-p <project dir>]\n    watch [server] [client] [debug] [-p <project dir>]\n\nRefer to docs/cli.md for the full API documentation\n");
}
//# sourceMappingURL=index.js.map