"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var fs_1 = tslib_1.__importDefault(require("fs"));
var path_1 = tslib_1.__importDefault(require("path"));
var assets_webpack_plugin_1 = tslib_1.__importDefault(require("assets-webpack-plugin"));
var html_webpack_plugin_1 = tslib_1.__importDefault(require("html-webpack-plugin"));
var server_1 = require("../runtime/server");
var assets_1 = require("../runtime/server/assets");
var ts_loader_config_1 = require("./ts-loader-config");
var getPlugins = function (buildConfig) { return [
    new assets_webpack_plugin_1.default({
        filename: path_1.default.relative(process.cwd(), server_1.getAssetsFile(buildConfig.OUTPUT)),
        processOutput: function (assets) {
            assets_1.updateAssetLocations(assets);
            return JSON.stringify(assets);
        },
    }),
]; };
/**
 * Base webpack config for the client that is used both in development and production
 * - Compile SCSS to CSS and extract into external assets
 * - Create assets.json that maps the created assets to their locations
 */
var clientBaseConfig = function (options) {
    var _a = options.buildConfig, CLIENT_ENTRY = _a.CLIENT_ENTRY, OUTPUT = _a.OUTPUT, CLIENT_POLYFILLS = _a.CLIENT_POLYFILLS, PUBLIC_PATH = _a.PUBLIC_PATH, SERVER_PUBLIC_DIR = _a.SERVER_PUBLIC_DIR;
    var mainEntry = [CLIENT_ENTRY];
    /*
        https://github.com/webpack/webpack/issues/6647
        You should not have a entrypoint for vendor. It's not an entry.
        If you want to run two modules in order at an entrypoint, use an array.
    */
    if (CLIENT_POLYFILLS && fs_1.default.existsSync(CLIENT_POLYFILLS)) {
        mainEntry = [CLIENT_POLYFILLS, CLIENT_ENTRY];
    }
    var entry = {
        main: mainEntry,
    };
    if (options.buildConfig.ADDITIONAL_CIENT_ENTRIES) {
        Object.assign(entry, options.buildConfig.ADDITIONAL_CIENT_ENTRIES);
    }
    var plugins = getPlugins(options.buildConfig);
    var resolvePlugins = [];
    if (SERVER_PUBLIC_DIR) {
        var indexHtml = path_1.default.resolve(SERVER_PUBLIC_DIR, 'index.html');
        if (fs_1.default.existsSync(indexHtml)) {
            plugins.push(new html_webpack_plugin_1.default({
                inject: true,
                template: indexHtml,
            }));
        }
    }
    return {
        entry: entry,
        module: {
            rules: [ts_loader_config_1.getTypeScriptWebpackRule(plugins, resolvePlugins, options, 'client')],
        },
        output: {
            path: OUTPUT,
            publicPath: PUBLIC_PATH,
        },
        plugins: plugins,
        resolve: {
            plugins: resolvePlugins,
        },
    };
};
exports.default = clientBaseConfig;
//# sourceMappingURL=webpack.client.js.map