"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var child_process_1 = require("child_process");
var dotenv_1 = tslib_1.__importDefault(require("dotenv"));
var express_1 = tslib_1.__importDefault(require("express"));
var http_proxy_middleware_1 = tslib_1.__importDefault(require("http-proxy-middleware"));
var path_1 = tslib_1.__importDefault(require("path"));
var webpack_1 = tslib_1.__importDefault(require("webpack"));
var build_1 = require("../build/build");
var server_1 = require("../runtime/server/server");
var network_1 = require("../runtime/util/network");
var dev_1 = require("../server/dev");
var webpack_2 = require("../util/webpack");
var restartServer = function (buildConfig, port, projectDir, oldServer) {
    if (oldServer) {
        oldServer.kill();
    }
    // When running in local dev, we have a different process.cwd() than
    // when running in production. This allows static files and such to resolve
    return child_process_1.fork(path_1.default.resolve(buildConfig.OUTPUT, 'server.js'), [], {
        env: tslib_1.__assign(tslib_1.__assign({}, process.env), { LOAD_DEFAULT_ASSETS: 'true', PORT: port.toString(), PROJECT_DIR: projectDir }),
    });
};
function watchServer(log, buildConfig) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var hostPort, devServerPort, devServer, devServerAvailable, serverConfig, serverCompiler, watching, app, watchServer;
        var _this = this;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    dotenv_1.default.config({
                        path: path_1.default.join(buildConfig.BASE, '.env'),
                    });
                    return [4 /*yield*/, network_1.findFreePort(server_1.getPort(buildConfig.DEV_SERVER_PORT))];
                case 1:
                    hostPort = _a.sent();
                    return [4 /*yield*/, network_1.findFreePort(hostPort + 1)];
                case 2:
                    devServerPort = _a.sent();
                    serverConfig = build_1.getWebpackConfig(log, buildConfig, 'server', 'dev');
                    serverCompiler = webpack_1.default(serverConfig);
                    serverCompiler.hooks.invalid.tap('invalid', function () {
                        log.info('⭐  Server changed, rebuilding and restarting server...  ⭐');
                    });
                    watching = serverCompiler.watch({}, function (err, stats) {
                        if (!devServer) {
                            setTimeout(function () { return dev_1.openBrowser(log, hostPort); }, 2000);
                        }
                        if (err) {
                            log.error({ err: err }, 'Failed to compile');
                            return;
                        }
                        else {
                            var statsString = stats.toString(webpack_2.webpackStatsConfig);
                            log.info(statsString);
                            if (stats.hasErrors()) {
                                log.error('Stats has errors');
                                return;
                            }
                        }
                        devServer = restartServer(buildConfig, devServerPort, buildConfig.BASE, devServer);
                        setTimeout(function () {
                            devServerAvailable = network_1.waitForConnection(devServerPort);
                        }, 100);
                    });
                    app = express_1.default();
                    app.use(dev_1.getHotReloadMiddleware(log, buildConfig));
                    app.use(function (_req, _res, next) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                        return tslib_1.__generator(this, function (_a) {
                            switch (_a.label) {
                                case 0: return [4 /*yield*/, devServerAvailable];
                                case 1:
                                    _a.sent();
                                    next();
                                    return [2 /*return*/];
                            }
                        });
                    }); });
                    app.use(http_proxy_middleware_1.default('http://localhost:' + devServerPort));
                    return [4 /*yield*/, new Promise(function (resolve) {
                            var server = app.listen(hostPort, function () {
                                resolve({
                                    app: app,
                                    close: function () {
                                        return Promise.all([
                                            new Promise(function (closeResolve) {
                                                return watching.close(function () {
                                                    closeResolve();
                                                });
                                            }),
                                            new Promise(function (closeResolve) { return server.close(function () { return closeResolve(); }); }),
                                        ]).then(function () {
                                            if (devServer) {
                                                devServer.kill();
                                            }
                                        });
                                    },
                                    server: server,
                                });
                            });
                        })];
                case 3:
                    watchServer = _a.sent();
                    app.set('server', watchServer);
                    return [2 /*return*/, watchServer];
            }
        });
    });
}
exports.default = watchServer;
//# sourceMappingURL=server.js.map