import { __rest } from "tslib";
import React from 'react';
import { createLocation } from 'history';
import { BrowserRouter, Route } from 'react-router-dom';
import { getTransferredState } from '../../client/get-transferred-state';
import { HelmetProvider } from 'react-helmet-async';
const locationOverrideFromSSR = getTransferredState('watchtower_hydrate_location');
/**
 * A wrapper around React routers Browser router
 * It takes care of hydrating error routes if they happened on the server.
 *
 * It also wraps your application in a HelmetProvider
 */
export const WatchtowerBrowserRouter = (_a) => {
    var { children } = _a, props = __rest(_a, ["children"]);
    const initialLocation = React.useRef();
    return (React.createElement(HelmetProvider, null,
        React.createElement(BrowserRouter, Object.assign({}, props),
            React.createElement(Route, { render: (routeProps) => {
                    if (!initialLocation.current) {
                        initialLocation.current = routeProps.location;
                    }
                    // While still on the initial route (which was hydrated), override the url
                    // if an override came from the server
                    const locationOverride = initialLocation.current === routeProps.location &&
                        locationOverrideFromSSR
                        ? createLocation(locationOverrideFromSSR)
                        : undefined;
                    if (locationOverride) {
                        return React.createElement(Route, { location: locationOverride }, children);
                    }
                    return React.Children.only(children);
                } }))));
};
WatchtowerBrowserRouter.displayName = 'WatchtowerBrowserRouter';
//# sourceMappingURL=watchtower-browser-router.js.map