import { __awaiter } from "tslib";
import open from 'open';
import webpack from 'webpack';
import AssetsPlugin from 'assets-webpack-plugin';
import webpackDevMiddleware from 'webpack-dev-middleware';
import webpackHotMiddleware from 'webpack-hot-middleware';
import { getWebpackConfig } from '../build/build';
export const getHotReloadMiddleware = (log, buildConfig) => {
    const config = getWebpackConfig(log, buildConfig, 'client', 'dev');
    if (!config) {
        throw new Error('Unable to load webpack config');
    }
    if (config.plugins) {
        config.plugins = config.plugins.filter(plugin => !(plugin instanceof AssetsPlugin));
    }
    const compiler = webpack(config);
    const dev = webpackDevMiddleware(compiler, {
        // do not serve index.html on / route
        // https://github.com/webpack/webpack-dev-middleware/issues/153
        index: 'foobar',
        logLevel: 'silent',
        publicPath: buildConfig.PUBLIC_PATH,
        stats: 'errors-only',
    });
    const hot = webpackHotMiddleware(compiler);
    return [dev, hot];
};
export const openBrowser = (log, port) => __awaiter(void 0, void 0, void 0, function* () {
    if (process.env.NODE_ENV === 'test') {
        return;
    }
    try {
        yield open(`http://localhost:${port}`, { url: true });
    }
    catch (e) {
        log.warn(e, 'Opening browser failed');
    }
});
//# sourceMappingURL=dev.js.map