import fs from 'fs';
import path from 'path';
import TsconfigPathsPlugin from 'tsconfig-paths-webpack-plugin';
var disableCaching = String(process.env.BUILD_CACHE_DISABLED).toLowerCase() === 'true';
var disableTypeCheck = process.env.DISABLE_TYPE_CHECKING === 'true';
var cacheLoader = function (cacheDirectory) {
    return {
        loader: 'cache-loader',
        options: {
            cacheDirectory: cacheDirectory,
        },
    };
};
export function getTypeScriptWebpackRule(_plugins, resolvePlugins, options, buildTarget) {
    var configFile = getTsConfigFile(buildTarget, options.buildConfig);
    options.log.info("Target " + buildTarget + " using ts config file: " + configFile);
    var tsConfigPathsPluginConfig = {};
    var tsLoaderOptions = {
        compilerOptions: {
            noEmit: false,
        },
        transpileOnly: disableTypeCheck,
        experimentalWatchApi: true,
        projectReferences: true,
    };
    if (configFile) {
        tsConfigPathsPluginConfig.configFile = configFile;
        tsLoaderOptions.configFile = configFile;
    }
    resolvePlugins.push(new TsconfigPathsPlugin(tsConfigPathsPluginConfig));
    var babelConfig = getBabelConfigFile(buildTarget, options.buildConfig);
    if (babelConfig) {
        options.log.info("Using babel config: " + babelConfig);
    }
    var tsLoader = {
        loader: 'ts-loader',
        options: tsLoaderOptions,
    };
    var babelLoader = {
        loader: 'babel-loader',
        options: {
            extends: babelConfig,
        },
    };
    return {
        test: /\.tsx?$/,
        use: !disableCaching
            ? [cacheLoader(options.cacheDirectory), babelLoader, tsLoader]
            : [babelLoader, tsLoader],
    };
}
export function getTsConfigFile(buildTarget, buildConfig) {
    return ((buildTarget === 'server' ? buildConfig.TS_CONFIG_SERVER : buildConfig.TS_CONFIG_CLIENT) ||
        'tsconfig.json');
}
export function getBabelConfigFile(buildTarget, buildConfig) {
    var babelConfigFile = (buildTarget === 'server'
        ? buildConfig.BABEL_CONFIG_SERVER
        : buildConfig.BABEL_CONFIG_CLIENT) || '.babelrc';
    var babelConfig = path.resolve(buildConfig.BASE, babelConfigFile);
    if (!fs.existsSync(babelConfig)) {
        babelConfig = path.resolve(process.cwd(), babelConfigFile);
        if (!fs.existsSync(babelConfig)) {
            babelConfig = undefined;
        }
    }
    return babelConfig;
}
//# sourceMappingURL=ts-loader-config.js.map