import fs from 'fs';
import path from 'path';
import AssetsPlugin from 'assets-webpack-plugin';
import HtmlWebpackPlugin from 'html-webpack-plugin';
import { getAssetsFile } from '../runtime/server';
import { updateAssetLocations } from '../runtime/server/assets';
import { getTypeScriptWebpackRule } from './ts-loader-config';
var getPlugins = function (buildConfig) { return [
    new AssetsPlugin({
        filename: path.relative(process.cwd(), getAssetsFile(buildConfig.OUTPUT)),
        processOutput: function (assets) {
            updateAssetLocations(assets);
            return JSON.stringify(assets);
        },
    }),
]; };
/**
 * Base webpack config for the client that is used both in development and production
 * - Compile SCSS to CSS and extract into external assets
 * - Create assets.json that maps the created assets to their locations
 */
var clientBaseConfig = function (options) {
    var _a = options.buildConfig, CLIENT_ENTRY = _a.CLIENT_ENTRY, OUTPUT = _a.OUTPUT, CLIENT_POLYFILLS = _a.CLIENT_POLYFILLS, PUBLIC_PATH = _a.PUBLIC_PATH, SERVER_PUBLIC_DIR = _a.SERVER_PUBLIC_DIR;
    var mainEntry = [CLIENT_ENTRY];
    /*
        https://github.com/webpack/webpack/issues/6647
        You should not have a entrypoint for vendor. It's not an entry.
        If you want to run two modules in order at an entrypoint, use an array.
    */
    if (CLIENT_POLYFILLS && fs.existsSync(CLIENT_POLYFILLS)) {
        mainEntry = [CLIENT_POLYFILLS, CLIENT_ENTRY];
    }
    var entry = {
        main: mainEntry,
    };
    if (options.buildConfig.ADDITIONAL_CIENT_ENTRIES) {
        Object.assign(entry, options.buildConfig.ADDITIONAL_CIENT_ENTRIES);
    }
    var plugins = getPlugins(options.buildConfig);
    var resolvePlugins = [];
    if (SERVER_PUBLIC_DIR) {
        var indexHtml = path.resolve(SERVER_PUBLIC_DIR, 'index.html');
        if (fs.existsSync(indexHtml)) {
            plugins.push(new HtmlWebpackPlugin({
                inject: true,
                template: indexHtml,
            }));
        }
    }
    return {
        entry: entry,
        module: {
            rules: [getTypeScriptWebpackRule(plugins, resolvePlugins, options, 'client')],
        },
        output: {
            path: OUTPUT,
            publicPath: PUBLIC_PATH,
        },
        plugins: plugins,
        resolve: {
            plugins: resolvePlugins,
        },
    };
};
export default clientBaseConfig;
//# sourceMappingURL=webpack.client.js.map